/*
 * Copyright (c), Lawrence Foard, 1992
 *
 * Developed under contract by Virtual World Information systems.
 * Questions and suggestions about this package may be directed to:
 * lfoard@turing.org, entropy@world.std.com
 * 
 * Those without e-mail access can call the VWIS Linux support BBS
 * at 1-508-793-9568 and send e-mail to entropy,
 * or send snail mail to:
 *
 * Virtual World Information Systems
 * 217 Park Ave.
 * Box #108
 * Worcester MA 01609
 *
 * This code may be used under the terms of the GNU license.
 * Other licensing agreements may be negotiated with VWIS.
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <termios.h>
#include <linux/stat.h> /* for some reason sys/stat.h seems to be broken in 2.2.2+.97 */
#include "head.h"

extern int errno;

int do_ioctl(PROCESS *proc,unsigned int fd,int command,int data)
 {
  switch(command)
   {
    case 0x5401: /* TCGETA */
     return(ioctl(fd,TCGETA,&proc->prog_area[data]));
    case 0x5402: /* TCSETA */
     return(ioctl(fd,TCSETA,&proc->prog_area[data]));
    case 0x5403: /* TCSETAW */
     return(ioctl(fd,TCSETAW,&proc->prog_area[data]));
    case 0x5404: /* TCSETAF */
     return(ioctl(fd,TCSETAF,&proc->prog_area[data]));
    case 0x5722: /* ??? */
     errno=1;
     return -1;
    default: 
     fprintf(stderr,"Unrecognized ioctl fd=%d %4.4X [%c] [%d]\n",fd,command,command>>8,command>>8);
     exit(1);
   }
 }
 
#define X_NDELAY 0004
#define X_APPEND 0010
#define X_SYNCW  0020
#define X_CREAT  0400
#define X_TRUNC  1000

int xtol(int flags)
 {
  int l=0;
  if (flags==-1)
   return -1;
  l=flags & 3;
  if (flags & X_NDELAY)
   l|=O_NDELAY;
  if (flags & X_APPEND)
   l|=O_APPEND;
  if (flags & X_SYNCW)
   l|=0;
  if (flags & X_CREAT)
   l|=O_CREAT;
  if (flags & X_TRUNC)
   l|=O_TRUNC;
  return l; 
 }
 
int ltox(int flags)
 { 
  int l=0;
  if (flags==-1)
   return -1;
  l=flags & 3;
  if (flags & O_NDELAY)
   l|=X_NDELAY;
  if (flags & O_APPEND)
   l|=X_APPEND;
  if (flags & O_CREAT)
   l|=X_CREAT;
  if (flags & O_TRUNC)
   l|=X_TRUNC;
  return l;
 } 
 
int do_fcntl(PROCESS *proc,int fd,unsigned int command,int data)
 {
  switch(command)
   {
    case F_GETFL:
     return ltox(fcntl(fd,command,data));
    case F_SETFL:
     return fcntl(fd,command,xtol(data));
    default: 
     return fcntl(fd,command,data);
   }
 }

/* convert stat structure */

struct x_stat
 {
  short dev;
  short ino;
  unsigned short mode;
  short links;
  unsigned short uid;
  unsigned short gid;
  short rdev;
  unsigned int size;
  unsigned int atime;
  unsigned int mtime;
  unsigned int ctime;
 };

/* convert linux stat structure to xenix format and vice versa */ 
void *lst_to_xst(struct new_stat *stat,void *x_stat1)
 {
  struct x_stat *x_stat=(struct x_stat *) x_stat1;
  x_stat->dev=stat->st_dev;
  x_stat->ino=stat->st_ino;
  x_stat->mode=stat->st_mode;
  x_stat->links=stat->st_nlink;
  x_stat->uid=stat->st_uid;
  x_stat->gid=stat->st_gid;
  x_stat->rdev=stat->st_rdev;
  x_stat->size=stat->st_size;
  x_stat->atime=stat->st_atime;
  x_stat->mtime=stat->st_mtime;
  x_stat->ctime=stat->st_ctime;
  return &x_stat;
 }
 
struct new_stat *xst_to_lst(void *tmp)
 {
  struct x_stat *x_stat=tmp;
  static struct new_stat stat;
  stat.st_dev=x_stat->dev;
  stat.st_ino=x_stat->ino;
  stat.st_mode=x_stat->mode;
  stat.st_nlink=x_stat->links;
  stat.st_uid=x_stat->uid;
  stat.st_rdev=x_stat->rdev;
  stat.st_size=x_stat->size;
  stat.st_atime=x_stat->atime;
  stat.st_mtime=x_stat->mtime;
  stat.st_ctime=x_stat->ctime;
  return &stat;
 }
